@@ -3,9 +3,7 @@ mmod_dimension <- function(object){
 }
 
 .mmod_dimension <- function(dlq, datainfo){
-
   .mmod_dimensionPrimitive(dlq, datainfo$cont.names, datainfo$disc.names, datainfo$disc.levels)
-
 }
 
 .mmod_dimensionPrimitive <- function(dlq, cont.names, disc.names, disc.levels ){
@@ -14,17 +12,18 @@ mmod_dimension <- function(object){
   ddd <- dlq$discrete
   qqq <- dlq$quadratic
 
-  ##  cat(sprintf("CHK: linear generators: %s\n", toString(lll)))
-  ##cat(sprintf("CHK: cont.names: %s\n", toString(cont.names)))
-  
-  n.cont <- length(cont.names)
-  d.dim <- c(lapply(disc.levels, length), recursive=TRUE)
+##   cat(".mmod_dimensionPrimitive\n")
+##   cat(sprintf("CHK: linear generators: %s\n", toString(lll)))
+##   cat(sprintf("CHK: cont.names: %s\n", toString(cont.names)))
+   
+  n.cont   <- length(cont.names)
+  d.levels <- c(lapply(disc.levels, length), recursive=TRUE)
   
   ## discrete dimension
   ## ------------------
   ddd.num  <- lapply(ddd, match, disc.names)
-  disc.dim <- .loglinDim(ddd.num, d.dim)
-  ##cat(sprintf("CHK: disc.dim=%i\n", disc.dim))
+  disc.dim <- .loglinDim(ddd.num, d.levels)
+##   cat(sprintf("CHK: disc.dim=%i\n", disc.dim))
   
   ## linear dimension
   ## ----------------
@@ -33,33 +32,36 @@ mmod_dimension <- function(object){
   } else {
     lin.dim <- 0
     for (ii in seq_along(cont.names)){
-      cont <- cont.names[ii]
-      
-      idx  <- any(unlist(lapply(lll, function(xx) any(match(xx, cont)))))
-      #cat(sprintf("  .. cont=%s\n", cont))
-      #print(idx)
-      
-      if (is.na(idx)){
+      cvar <- cont.names[ii]
+      ##cat(sprintf("cvar=%s\n", cvar))
+      ## Find those linear generators containing 'cvar'
+      bbb <- unlist(lapply(lll, function(xx) any(match(xx, cvar))))
+      ##print(bbb)
+      if (is.na(any(bbb))){ ## If there are no such generators
         zzz <- 1
       } else {
-        aaa  <- lll[idx][!unlist(lapply(lll[idx], is.null))]      
-        aaa  <- lapply(aaa, function(xx) xx[-1])
-        if (length(aaa)>0){
-          aaa.num <- lapply(aaa, match, disc.names)
-          zzz     <- .loglinDim(aaa.num, d.dim) + 1
+        ## Find discrete generators corresponding to 'cvar'
+        ccc <- lapply(lll[which(bbb)], setdiff, cvar)        
+        ## A linear generator may consist of a 'cvar' itself; such generator must be removed
+        ccc <- ccc[ sapply(ccc,length) >0 ]
+        ##print(ccc)
+        if (length(ccc)>0){
+          ccc.num <- lapply(ccc, match, disc.names)
+          ##print(ccc.num)
+          ##print(d.levels)
+          zzz     <- .loglinDim(ccc.num, d.levels) + 1
+          ##print(zzz)
         }
       }
-      ##cat(sprintf("CHK:  .. cont=%10s dim=%i\n", cont, zzz))
+      ##cat(sprintf("CHK:  .. cvar=%10s dim=%i\n", cvar, zzz))
       lin.dim <- lin.dim + zzz    
     }
   }
-  
   ##cat(sprintf("CHK: lin.dim=%i\n", lin.dim))
 
   ## quadratic dimension
   ## -------------------
-  ##uuu      <- ugList(qqq, result="matrix")
-  uuu <- glist2adjMAT(qqq)
+  uuu      <- glist2adjMAT(qqq)
   quad.dim <- sum(uuu[upper.tri(uuu)]) + nrow(uuu)
   ##cat(sprintf("CHK: quad.dim=%i\n", quad.dim))
 
@@ -69,22 +71,37 @@ mmod_dimension <- function(object){
   
   ## saturated model dimension
   ## -------------------------
-##   sat.dim <- prod(d.dim-1) +  ## discrete 
-##     n.cont * prod(d.dim) +    ## linear 
-##       n.cont*(n.cont+1)/2     ## quadratic
-
-  sat.dim <- prod(d.dim)-1 +  ## discrete 
-    n.cont * prod(d.dim) +    ## linear 
-      n.cont*(n.cont+1)/2     ## quadratic
+  sat.dim <- prod(d.levels)-1 +  ## discrete 
+    n.cont * prod(d.levels) +    ## linear 
+      n.cont*(n.cont+1)/2        ## quadratic
   
   ## independence model dimension
   ## ----------------------------
-  i.dim <- sum(d.dim - 1) + 2*n.cont
+  i.dim <- sum(disc.dim - 1) + 2*n.cont
   
   c(mod.dim=mod.dim, sat.dim=sat.dim, i.dim=i.dim, df=sat.dim-mod.dim, idf=mod.dim-i.dim)
 }
 
 
+##       browser()
+##       idx  <- any(unlist(lapply(lll, function(xx) any(match(xx, cvar)))))
+##       cat(sprintf("  .. cvar=%s\n", cvar))
+##       if (is.na(idx)){
+##         zzz <- 1
+##       } else {
+##         aaa  <- lll[idx][!unlist(lapply(lll[idx], is.null))]      
+##         print(aaa)
+##         aaa  <- lapply(aaa, function(xx) xx[-1])
+##         print(aaa)
+##         if (length(aaa)>0){
+##           aaa.num <- lapply(aaa, match, disc.names)
+##           print(aaa.num)
+##           zzz     <- .loglinDim(aaa.num, disc.dim) + 1
+##         }
+##       }
+
+
+
 .loglinDim <- function(glistNUM, dtab){
 
   #print(glistNUM); print (dtab)
@@ -103,7 +120,7 @@ mmod_dimension <- function(object){
   if (max.g.size < 10)
     {
       zz    <- unlist(lapply(glistNUM, .subsets), recursive=FALSE)
-      unzz  <- unique(zz)
+      unzz  <- unique.default(zz)
     }
   else 
     {
